#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl server command
'''
from cloudcontrol.cli.exception import *
from sjrpc.core.exceptions import *
from cloudcontrol.cli.printer import Printer, color
from cloudcontrol.cli.command import RemoteCommand

class Command_server(RemoteCommand):
    '''Server manipulation command'''

    def __init__(self, cli, argv0):
        RemoteCommand.__init__(self, cli, argv0)
        self.set_usage("%prog <options>")
        self.add_option("-c", action="store_true", dest="cache",
                        help="show server cache")
        self.add_option("-f", action="store_true", dest="functions",
                        help="show server functions")
        self.add_option("-v", action="store_true", dest="version",
                        help="show server version")

    def __call__(self, argv):
        self.parse_args(argv)
        if len(self.args) > 0:
            self.printer.out(self.usage())
        elif self.options.cache:
            self.show_cache()
        elif self.options.functions:
            self.show_functions()
        elif self.options.version:
            self.show_version()
        else:
            self.printer.out(self.usage())

    def remote_functions(self):
        return set(('dbstats', 'version'))

    def show_functions(self):
        try:
            for cmds in self.cli.rpc.call("functions"):
                self.printer.out("%s"%cmds["name"])
        except RpcError as e:
            raise cmdError("RPCError: %s"%str(e))

    def show_cache(self):
        try:
            d = self.cli.rpc.call("dbstats")
            for i,v in d.items():
                self.printer.out("%s: %s"%(i,v))
        except RpcError as e:
            raise cmdError("RPCError: %s"%str(e))

    def show_version(self):
        try:
            self.printer.out(self.cli.rpc.call("version"))
        except RpcError as e:
            raise cmdError("RPCError: %s"%str(e))
